;******************************************************************************
;                                                                             *
;    Filename:		MBLiBC.asm                                                *
;    Date:          17FEB05                                                   *
;    File Version:  v1.0                                                      *
;
;* Software License Agreement													
;*																				
;* The software supplied herewith by Microchip Technology Incorporated			
;* (the "Company") for its PICmicro Microcontroller is intended and			
;* supplied to you, the Company's customer, for use solely and					
;* exclusively on Microchip PICmicro Microcontroller products. The				
;* software is owned by the Company and/or its supplier, and is					
;* protected under applicable copyright laws. All rights are reserved.			
;* Any use in violation of the foregoing restrictions may subject the			
;* user to criminal sanctions under applicable laws, as well as to				
;* civil liability for the breach of the terms and conditions of this			
;* license.																		
;*																				
;* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,			
;* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED			
;* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A					
;* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,			
;* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR					
;* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.							
;
;                                                                             *
;    Author:        Scott Dearborn                                            *
;    Company:       Microchip Technology, Inc.                                *
;                                                                             * 
;******************************************************************************
;                                                                             *
;    Files required:         P18F2410.INC                                     *
;                            18F2410.LKR                                      *
;                                                                             *
;******************************************************************************

	LIST P=18F2410, F=INHX32 ;directive to define processor and file format
	#include <P18F2410.INC>	 ;processor specific variable definitions

;******************************************************************************
;Configuration bits
; The __CONFIG directive defines configuration data within the .ASM file.
; The labels following the directive are defined in the P18F2410.INC file.
; The PIC18F2410 Data Sheet explains the functions of the configuration bits.

      __CONFIG  H'300000', H'FF'
      __CONFIG  _CONFIG1H, _IESO_OFF_1H & _FCMEN_OFF_1H & _OSC_INTIO7_1H
      __CONFIG  _CONFIG2L, _PWRT_ON_2L & _BOREN_OFF_2L & _BORV_42_2L
      __CONFIG  _CONFIG2H, _WDT_OFF_2H & _WDTPS_32768_2H
      __CONFIG  H'300004', H'FF'
      __CONFIG  _CONFIG3H, _MCLRE_ON_3H & _PBADEN_ON_3H & _CCP2MX_PORTC_3H
      __CONFIG  _CONFIG4L, _DEBUG_OFF_4L & _LVP_OFF_4L & _ENHCPU_OFF_4L & _STVREN_OFF_4L
      __CONFIG  H'300007', H'FF'
      __CONFIG  _CONFIG5L, _CP0_OFF_5L & _CP1_OFF_5L
      __CONFIG  _CONFIG5H, _CPB_OFF_5H
      __CONFIG  _CONFIG6L, _WRT0_OFF_6L & _WRT1_OFF_6L
      __CONFIG  _CONFIG6H, _WRTC_OFF_6H & _WRTB_OFF_6H
      __CONFIG  _CONFIG7L, _EBTR0_OFF_7L & _EBTR1_OFF_7L
      __CONFIG  _CONFIG7H, _EBTRB_OFF_7H

;******************************************************************************
;Constant definitions
;BC_STATUS bits
BAT_CONNECT			EQU	0x00
PRECOND				EQU	0x01
FAST_CHARGE			EQU	0x02
CHARGE_COMPLETE		EQU	0x03
TEMP_FAULT			EQU	0x04
CHARGE_SUSPEND		EQU	0x05
TIMER_FAULT			EQU	0x06
OVER_VOLTAGE		EQU	0x07

;BAY_SELECT bits
CELL_TEMP_MONITOR	EQU	0x00
BAY					EQU	0x06
UPDATE				EQU	0x07

;Battery Voltages
VBAT43H				EQU	0x11	;overvoltage threshold
VBAT42H				EQU	0x10	;battery detection threshold and regulation
VBAT42L				EQU	0x68	;battery detection threshold and regulation
VBAT40H				EQU	0x0F	;battery recharge threshold
VBAT40L				EQU	0xA0	;battery recharge threshold
VBAT28H				EQU	0x0B	;preconditioning threshold
VBAT28L				EQU	0x00	;preconditioning threshold

;Battery Temperatures
TEMP_HIGH			EQU	0x43	;50 degrees Celcius
TEMP_HIGH_HYS		EQU	0x3E
TEMP_LOW			EQU	0x11	;0 degrees Celcius
TEMP_LOW_HYS		EQU	0x16

;Charge Currents
PRECOND_CURRENT		EQU	0x19	;200 mA
FAST_CURRENT		EQU	0xFB	;2000 mA
TERMINATION_CURRENT	EQU	0x0C	;100 mA

;Timer
PRECOND_TIME		EQU	0x16	;22 minutes
ELAPSED_TIME		EQU	0x05	;5 hours

;LED mask 
LED1_ON				EQU	0x20
LED2_ON				EQU	0x01
C1MINUS_IO			EQU	0x01
C2MINUS_IO			EQU	0x02

;SMBus 
SDA1				EQU	0x06
SDA2				EQU	0x07
PS700ADDRESS_W		EQU	0xC2
PS700ADDRESS_R		EQU	0xC3
PS700EEPROM			EQU	0x00
PS700REGISTERS		EQU	0x04
PS700VPACKCTRL		EQU	0x4E
PS700VPACKH			EQU	0x4D
PS700VPACKL			EQU	0x4C
PS700VGAINH			EQU	0x97
PS700VGAINL			EQU	0x96
PS700ITCTRL			EQU	0x46
PS700ITEMPH			EQU	0x45
PS700ITEMPL			EQU	0x44
PS700ITGAINH		EQU	0x9F
PS700ITGAINL		EQU	0x9E
PS700READCNT		EQU	0x05

;ADC channels
VSNS1				EQU	0x2B
VSNS2				EQU	0x23
TSNS1				EQU	0x2F
TSNS2				EQU	0x27
ADC_VGAINH			EQU	0x04
ADC_VGAINL			EQU	0xE2
ADC_TGAINH			EQU	0x00
ADC_TGAINL			EQU	0x19

;******************************************************************************
;Variable definitions

		UDATA_ACS	0x000	
BC_STATUS1	RES	1	;battery charger bay 1 status		00
LOOP_COUNT1	RES	1	;loop counter bay 1					01
SECOND1		RES	1	;second counter bay 1				02
MINUTE1		RES	1	;minute counter bay 1				03
HOUR1		RES	1	;hour counter bay 1					04
VBAT1H		RES	1	;battery voltage bay 1 high byte	05
VBAT1L		RES	1	;battery voltage bay 1 low byte		06
BTEMP1		RES	1	;battery temperature bay 1			07
LED1		RES	1	;charge status bay 1				08
C1MINUS		RES	1	;charge fault bay 1					09
TEMP_SEC1	RES	1	;temporary second counter bay 1		0A
TEMP_MIN1	RES	1	;temporary minute counter bay 1		0B
TEMP_HOUR1	RES	1	;temporary hour counter bay 1		0C
TEMP_DC1H	RES	1	;temporary duty cycle bay 1			0D
TEMP_DC1L	RES	1	;temporary duty cycle bay 1			0E
AD_SEL1		RES	1	;ADC channel select bay 1			0F

BC_STATUS2	RES	1	;battery charger bay 2 status		10
LOOP_COUNT2	RES	1	;loop counter bay 2					11
SECOND2		RES	1	;second counter bay 2				12
MINUTE2		RES	1	;minute counter bay 2				13
HOUR2		RES 1	;hour counter bay 2					14
VBAT2H		RES	1	;battery voltage bay 2 high byte	15
VBAT2L		RES	1	;battery voltage bay 2 low byte		16
BTEMP2		RES	1	;battery temperature bay 2			17
LED2		RES	1	;charge status bay 2 				18
C2MINUS		RES	1	;charge fault bay 2 				19
TEMP_SEC2	RES	1	;temporary second counter bay 2		1A
TEMP_MIN2	RES	1	;temporary minute counter bay 2		1B
TEMP_HOUR2	RES	1	;temporary hour counter bay 2		1C
TEMP_DC2H	RES	1	;temporary duty cycle bay 2			1D
TEMP_DC2L	RES	1	;temporary duty cycle bay 2			1E
AD_SEL2		RES	1	;ADC channel select bay 2			1F

SMA_VGAINH	RES	1	;variable for PS700 voltage reading	20
SM_VGAINH	RES	1	;variable for PS700 voltage reading	21
SMA_VGAINL	RES	1	;variable for PS700 voltage reading	22
SM_VGAINL	RES	1	;variable for PS700 voltage reading	23
SMA_VCTRL	RES	1	;variable for PS700 voltage reading	24
SM_VCTRL	RES	1	;variable for PS700 voltage reading	25
SMA_VPACKH	RES	1	;variable for PS700 voltage reading	26
SM_VPACKH	RES	1	;variable for PS700 voltage reading	27
SMA_VPACKL	RES	1	;variable for PS700 voltage reading	28
SM_VPACKL	RES	1	;variable for PS700 voltage reading	29

SMA_TGAINH	RES	1	;variable for PS700 temp. reading	2A
SM_TGAINH	RES	1	;variable for PS700 temp. reading	2B
SMA_TGAINL	RES	1	;variable for PS700 temp. reading	2C
SM_TGAINL	RES	1	;variable for PS700 temp. reading	2D
SMA_TCTRL	RES	1	;variable for PS700 temp. reading	2E	
SM_TCTRL	RES	1	;variable for PS700 temp. reading	2F
SMA_TEMPH	RES	1	;variable for PS700 temp. reading	30
SM_TEMPH	RES	1	;variable for PS700 temp. reading	31
SMA_TEMPL	RES	1	;variable for PS700 temp. reading	32
SM_TEMPL	RES	1	;variable for PS700 temp. reading	33

SM_LOOP_CNT	RES	1	;variable for PS700 reading count	34
SM_BANK_SEL	RES	1	;variable for PS700 bank select		35

RES3		RES	1	;multiply result register high byte	36
RES2		RES	1	;multiply result register			37
RES1		RES	1	;multiply result register			38
RES0		RES	1	;multiply result register low byte	39

ARG1H		RES	1	;multiply argument register			3A
ARG1L		RES	1	;multiply argument register			3B
ARG2H		RES	1	;multiply argument register			3C
ARG2L		RES	1	;multiply argument register			3D

BAY_SELECT	RES	1	;bay selection						3E
VBAT_TEMPH	RES	1	;temporary battery voltage high byte3F
VBAT_TEMPL	RES	1	;temporary battery voltage low byte	40
BTEMP_TEMP	RES	1	;temporary battery temperature		41
LED_TEMP	RES	1	;temporary LED value				42
TIME_TEMP	RES	1	;temporary mode time				43
TEMP_VAR1	RES	1	;temporary variable					44
COMP_TEMP	RES	1	;temporary comparator output		45
WREG_TEMP	RES	1	;variable in RAM for context saving 46
STATUS_TEMP	RES	1	;variable in RAM for context saving	47
BSR_TEMP	RES	1	;variable in RAM for context saving	48

;******************************************************************************
;Reset vector
; This code will start executing when a reset occurs.

RESET_VECTOR	CODE	0x0000

		goto	Main				;go to start of main code

;******************************************************************************
;High priority interrupt vector
; This code will start executing when a high priority interrupt occurs or
; when any interrupt occurs if interrupt priorities are not enabled.

HI_INT_VECTOR	CODE	0x0008

		bra	HighInt		;go to high priority interrupt routine

;******************************************************************************
;Low priority interrupt vector
; This code will start executing when a low priority interrupt occurs.
; This code can be removed if low priority interrupts are not used.

LOW_INT_VECTOR	CODE	0x0018

		bra	LowInt		;go to low priority interrupt routine

;******************************************************************************
;High priority interrupt routine
; The high priority interrupt code is placed here.
; This code is not enabled

		CODE

HighInt:
		btfss	CMCON,C1OUT,0
		goto	CHECK_BAY2_OV
		bcf		TRISA,4,0
		bsf		BC_STATUS1,OVER_VOLTAGE,0
CHECK_BAY2_OV
		btfss	CMCON,C2OUT,0
		goto	EXIT_HISR
		bcf		TRISA,5,0
		bsf		BC_STATUS2,OVER_VOLTAGE,0
EXIT_HISR
		bcf		PIR2,CMIF,0
		retfie	FAST

;******************************************************************************
;Low priority interrupt routine
; The low priority interrupt code is placed here.
; This code can be removed if low priority interrupts are not used.

LowInt:
		movff	STATUS,STATUS_TEMP	;save STATUS register
		movff	WREG,WREG_TEMP		;save working register
		movff	BSR,BSR_TEMP		;save BSR register

		btfss	PIR1,TMR1IF,0
		goto	EXIT_LISR
		bcf		PIR1,TMR1IF,0
		btfss	BAY_SELECT,BAY,0
		goto	BAY1
		bsf		PORTC,SDA2,0	 	;set SMBus data to bay 2
		bcf		PORTC,SDA1,0
		btg		BAY_SELECT,BAY,0	
		bsf		BAY_SELECT,UPDATE,0
		lfsr	FSR0,0x010			;set indirect addressing to bay 2 status
		lfsr	FSR1,0xFBB			;set indirect addressing to CCPR2L
		call	UPDATE_VARIABLES
		goto	EXIT_LISR

BAY1
		bsf		PORTC,SDA1,0		;set SMBus data to bay 1
		bcf		PORTC,SDA2,0
		btg		BAY_SELECT,BAY,0	
		bsf		BAY_SELECT,UPDATE,0
		lfsr	FSR0,0x000			;set indirect addressing to bay 1 status
		lfsr	FSR1,0xFBE			;set indirect addressing to CCPR1L
		call	UPDATE_VARIABLES

EXIT_LISR
		movff	BSR_TEMP,BSR		;restore BSR register
		movff	WREG_TEMP,WREG		;restore working register
		movff	STATUS_TEMP,STATUS	;restore STATUS register
		retfie

;******************************************************************************
;Start of main program
; The main program code is placed here.

Main:

;******************************************************************************
;Initialize oscillator
		movlw	0x60					;set internal oscillator to 4MHz
		movwf	OSCCON,0
		btfss	OSCCON,IOFS,0			;frequency stable?
		goto	$-2						;no, wait

;******************************************************************************
;Initialize ports
		movlw	0x03				 
		movwf	LATA,0					;set OV outputs 
		movlw	0x03					;set ADC reference to VDD and VSS
		movwf	ADCON1,0				;set AN0-AN4,AN8-AN11 to analog inputs
		movlw	0x08				
		movwf	TRISA					;set C1+ to input

		movlw	0x21
		movwf	LATB,0					;set VBAT outputs
		movlw	0x1E
		movwf	TRISB,0					;set voltage and temperature ADC to inputs

		movlw	0x40
		movwf	LATC,0					;set port C output latch to SDA1 on
		movlw	0x18					
		movwf	TRISC,0					;set SDA and SCL inputs

;******************************************************************************
;Initialize ADC
		bsf		ADCON0,ADON,0			;turn on ADC
		movlw	0xBC					;set result right justified, acquisition
		movwf	ADCON2,0				;time 20TAD, conversion clock FOSC/4

;******************************************************************************
;Initialize comparators
		movlw	0xC7					;power on comparator reference, output
		movwf	CVRCON,0				;to RA2, reference voltage equals 2.34V
		movlw	0x33					;invert both comparator outputs, two
		movwf	CMCON,0					;independent comparators with outputs 

;******************************************************************************
;Initialize PWM
		clrf	CCPR1L,0				;clear current command 1
		clrf	CCPR2L,0				;clear current command 2
		movlw	0x0C
		movwf	CCP1CON,0				;set CCP1 to PWM mode
		movwf	CCP2CON,0				;set CCP2 to PWM mode
		movlw	0xF9
		movwf	PR2,0					;set PWM period to 250 uS (4 kHz)
		bsf		T2CON,TMR2ON,0			;turn on timer 2, PWM time base

;******************************************************************************
;Initialize I2C
		bsf		SSPSTAT,SMP,0			;slew rate control disabled
		bsf		SSPCON1,SSPM3,0			;I2C master mode in hardware
		bsf		SSPCON1,SSPEN,0			;enable SSP module,
										;SDA and SCL configured as serial port pins
		movlw	0x09					;set I2C clock rate to 100kHz 
		movwf	SSPADD,0				;clock rate = Fosc / (4*(SSPADD+1))

;******************************************************************************
;Initialize timers - used for TEMP_FAULT timers and control loop
		bsf		T1CON,T1CKPS0			;set control period to 1S (2 * 524 mS)
		bsf		T1CON,T1CKPS1			;set control period to 1S (2 * 524 mS)

;******************************************************************************
;Initialize RAM variables
		lfsr	FSR0,0x000
NEXT	clrf	POSTINC0				;clear RAM
		btfss	FSR0L,7
		goto	NEXT

		movlw	LED1_ON
		movwf	LED1,0					;store LED1 mask at LED1
		movlw	LED2_ON
		movwf	LED2,0					;store LED2 mask at LED2
		movlw	C1MINUS_IO
		movwf	C1MINUS,0				;store LED3 mask at LED3
		movlw	C2MINUS_IO
		movwf	C2MINUS,0				;store LED4 mask at LED4
		movlw	VSNS1					;set ADC channel select 1 (AD_SEL1)
		movwf	AD_SEL1,0				;to bay 1 voltage sense (AN10)
		movlw	VSNS2					;set ADC channel select 2 (AD_SEL2)
		movwf	AD_SEL2,0				;to bay 2 voltage sense (AN8)
		movlw	PS700VGAINH
		movwf	SMA_VGAINH
		movlw	PS700VGAINL
		movwf	SMA_VGAINL
		movlw	PS700VPACKCTRL
		movwf	SMA_VCTRL
		movlw	PS700VPACKH
		movwf	SMA_VPACKH
		movlw	PS700VPACKL
		movwf	SMA_VPACKL
		movlw	PS700ITCTRL
		movwf	SMA_TCTRL
		movlw	PS700ITEMPH
		movwf	SMA_TEMPH
		movlw	PS700ITEMPL
		movwf	SMA_TEMPL
		movlw	PS700ITGAINH
		movwf	SMA_TGAINH
		movlw	PS700ITGAINL
		movwf	SMA_TGAINL

		bsf		BAY_SELECT,CELL_TEMP_MONITOR	;REMOVE THIS LINE OF CODE IF
												;CELL TEMPERATURE MONITORING
												;IS NOT REQUIRED

;******************************************************************************
;Initialize interrupts
		bsf		RCON,IPEN,0				;enable priority interrupts
		bcf		PIR1,TMR1IF,0			;clear timer 1 interrupt flag
		bsf		PIE1,TMR1IE,0			;unmask timer 1 interrupt
		bcf		IPR1,TMR1IP,0			;timer 1 interrupt low priority
		bsf		IPR2,CMIP,0				;comparator interrupt high priority
		bsf		INTCON,GIEH,0			;enable high priority interrupts
		bsf		INTCON,GIEL,0			;enable low priority interrupts

		bsf		T1CON,TMR1ON,0			;turn on timer 1
		bcf		SSPCON1,SSPEN,0
		nop
		bsf		SSPCON1,SSPEN,0
		btfss	TMR1H,0,0			;check communications watch-dog
		goto	$-2
		
POWER_UP_DELAY_2S
		btfss	SECOND1,1,0				;two seconds?
		goto	$-2						;no, wait

;**************************************************************************
;Main loop
LOOP	
		btfss	BAY_SELECT,UPDATE,0		;Loop time?
		goto	$-2						;no, wait
		bcf		BAY_SELECT,UPDATE,0		
		btfsc	INDF0,BAT_CONNECT,0		;is battery connected?
		goto	CHECK_PRECONDITIONING	;yes, check if in preconditioning 
		call	CHECK_BATTERY_PRESENT	;no, check for battery
		goto	LOOP					;go wait for next time

CHECK_PRECONDITIONING
		btfsc	INDF0,OVER_VOLTAGE,0	;battery removed?
		goto	BATTERY_REMOVED
		btfsc	INDF0,TEMP_FAULT,0		;temperature fault?
		goto	TEMPERATURE_FAULT		;yes, check for fault cleared
		btfsc	INDF0,TIMER_FAULT,0		;time out fault?
		goto	TIMEOUT_FAULT			;yes, check for battery removed
		btfss	INDF0,PRECOND,0			;in preconditioning phase?
		goto	CHECK_FAST_CHARGE		;no, check if in fast charge
		call	PRECONDITIONING			;yes, check for time to transition and faults
		goto	LOOP					;go wait for next time

CHECK_FAST_CHARGE
		btfss	INDF0,FAST_CHARGE,0
		goto	CHECK_RECHARGE
		call	UPDATE_CURRENT
		call	CHECK_CHARGE_COMPLETE
		goto	LOOP
		
CHECK_RECHARGE
		call	CHECK_BATTERY_CHARGE
		goto	LOOP

;******************************************************************************
;TEMPERATURE_FAULT charge suspended
TEMPERATURE_FAULT
		btfsc	INDF0,CHARGE_SUSPEND,0	;charge suspended?
		goto	CHECK_TEMP_FAULT_CLEARED;yes, check if temperature fault cleared
		movlw	0x09					
		comf	PLUSW0,W,0
		andwf	TRISA,F,0				;turn on fault LED (red)
		movlw	0x02
		movff	PLUSW0,TEMP_VAR1		
		movlw	0x0A					
		movff	TEMP_VAR1,PLUSW0		;save seconds counter
		movlw	0x03
		movff	PLUSW0,TEMP_VAR1
		movlw	0x0B
		movff	TEMP_VAR1,PLUSW0		;save minutes counter
		movlw	0x04
		movff	PLUSW0,TEMP_VAR1
		movlw	0x0C
		movff	TEMP_VAR1,PLUSW0		;save hours counter
		movlw	0x0D
		movff	INDF1,PLUSW0			;save charge current command
		clrf	POSTDEC1,0				;clear charge current command
		movlw	0x0E
		movff	INDF1,PLUSW0			;save charge current command
		movlw	0x0C
		movwf	INDF1,0					;clear charge current command
		bsf		INDF0,CHARGE_SUSPEND	;set charge suspend flag
		goto	LOOP					;go wait until next time
CHECK_TEMP_FAULT_CLEARED
		movlw	0x05					;copy battery voltage reading to
		movff	PLUSW0,VBAT_TEMPH		;temporary location (VBAT_TEMPH)
		movlw	VBAT43H
		cpfslt	VBAT_TEMPH,0			;battery voltage less than 4.3V?
		goto	BATTERY_REMOVED			;no, battery removed
		movlw	0x07					;copy battery temperature reading to
		movff	PLUSW0,BTEMP_TEMP		;temporary location (BTEMP_TEMP)
		movlw	TEMP_HIGH_HYS
		cpfslt	BTEMP_TEMP,0			;battery temperature in hot range?
		goto	LOOP					;no, wait to check next time
		movlw	TEMP_LOW_HYS
		cpfsgt	BTEMP_TEMP,0			;battery temperature in cold range?
		goto	LOOP					;no, wait to check next time
		movlw	0x0A					;yes,restore counters and 
		movff	PLUSW0,TEMP_VAR1		;charge current command
		movlw	0x02
		movff	TEMP_VAR1,PLUSW0		;restore seconds counter
		movlw	0x0B
		movff	PLUSW0,TEMP_VAR1
		movlw	0x03
		movff	TEMP_VAR1,PLUSW0		;restore minutes counter
		movlw	0x0C
		movff	PLUSW0,TEMP_VAR1
		movlw	0x04
		movff	TEMP_VAR1,PLUSW0		;restore hours counter
		bcf		INDF0,CHARGE_SUSPEND	;clear charge suspended flag
		bcf		INDF0,TEMP_FAULT		;clear temperature flag
		movlw	0x08
		movf	PLUSW0,W,0
		iorwf	PORTC,F,0				;ensure charge LED on (green)
		movlw	0x09
		movf	PLUSW0,W,0
		iorwf	TRISA,F,0				;activate OV comparator
		movlw	0x0D
		movff	PLUSW0,POSTDEC1			;restore charge current command
		movlw	0x0E
		movff	PLUSW0,INDF1			;restore charge current command
		goto	LOOP					;go wait until next time

;******************************************************************************
;TIMEOUT_FAULT charge terminated
TIMEOUT_FAULT
		movlw	0x09
		comf	PLUSW0,W,0
		andwf	TRISA,F,0				;turn on fault LED (red)
		clrf	POSTDEC1,0				;set charge current command to 0 mA
		movlw	0x0C
		movwf	INDF1,0
		movlw	0x05					;copy battery voltage reading to
		movff	PLUSW0,VBAT_TEMPH		;temporary location (VBAT_TEMPH)
		movlw	VBAT43H
		cpfslt	VBAT_TEMPH,0			;battery voltage less than 4.3V?
		goto	BATTERY_REMOVED			;no, battery removed
		goto	LOOP					;yes, battery still connected
		
;******************************************************************************
;BATTERY_REMOVED Subroutine
BATTERY_REMOVED
		clrf	INDF0					;clear all status flags
		movlw	0x08
		comf	PLUSW0,W,0
		andwf	PORTC,F,0				;turn off charge LED (green)
		movlw	0x08
		movf	PLUSW0,W,0
		iorwf	PORTB,F,0				;turn on 
		movlw	0x09
		comf	PLUSW0,W,0
		andwf	TRISA,F,0				;turn on fault LED (red)
		clrf	POSTDEC1,0				;clear charge current command
		movlw	0x0C
		movwf	INDF1,0					;clear charge current command
		goto	LOOP					;go wait until next time

;******************************************************************************
;CHECK_BATTERY_PRESENT Subroutine
CHECK_BATTERY_PRESENT
		movlw	0x05					;copy battery voltage reading to
		movff	PLUSW0,VBAT_TEMPH		;temporary location (VBAT_TEMPH)
		movlw	VBAT42H
		cpfsgt	VBAT_TEMPH,0			;battery voltage less than 4.2V?
		goto	CHECK_LOW_BYTE1
		return							;no, return no battery connected
CHECK_LOW_BYTE1
		cpfseq	VBAT_TEMPH,0
		goto	BATTERY_CONNECTED
		movlw	0x06					;copy battery voltage reading to
		movff	PLUSW0,VBAT_TEMPL		;temporary location (VBAT_TEMPL)
		movlw	VBAT42L
		cpfslt	VBAT_TEMPL,0			;battery voltage less than 4.2V?
		return							;no, return no battery connected
BATTERY_CONNECTED
		movlw	0x08
		movf	PLUSW0,W,0				;move LED mask to working register
		iorwf	PORTC,F,0				;turn on charge LED (green)
		iorwf	PORTB,F,0
		movlw	0x09
		movf	PLUSW0,W,0
		iorwf	TRISA,F,0				;activate OV comparator
		movlw	PRECOND_CURRENT			;set charge current command to 
		movwf	INDF1,0					;preconditioning current
		movlw	0x02
		clrf	PLUSW0					;clear seconds counter
		movlw	0x03
		clrf	PLUSW0					;clear minutes counter
		movlw	0x04
		clrf	PLUSW0					;clear hours counter
		bsf		INDF0,BAT_CONNECT,0
		bsf		INDF0,PRECOND,0
		return							;return battery connected,
										;in preconditioning

;******************************************************************************
;PRECONDITIONING Subroutine
PRECONDITIONING
		call	CHECK_FOR_FAULT
		btfsc	INDF0,TEMP_FAULT,0
		return
		btfsc	INDF0,TIMER_FAULT,0
		return
		movlw	0x05					;copy battery voltage reading to
		movff	PLUSW0,VBAT_TEMPH		;temporary location (VBAT_TEMPH)
		movlw	VBAT28H
		cpfsgt	VBAT_TEMPH,0			;battery voltage greater than 2.8V?
		return							;no, return in preconditioning
		movlw	0x02
		clrf	PLUSW0					;clear seconds counter
		movlw	0x03
		clrf	PLUSW0					;clear minutes counter
		movlw	0x04
		clrf	PLUSW0					;clear hours counter
		bcf		INDF0,PRECOND,0
		bsf		INDF0,FAST_CHARGE,0
		return							;return in fast charge

;******************************************************************************
;UPDATE_CURRENT Subroutine
UPDATE_CURRENT
		call	CHECK_FOR_FAULT
		btfsc	INDF0,TEMP_FAULT,0
		return
		btfsc	INDF0,OVER_VOLTAGE,0
		return
		movlw	0x05
		movff	PLUSW0,VBAT_TEMPH
		movlw	VBAT42H
		cpfslt	VBAT_TEMPH,0
		goto	SLOW_STEP
		movlw	FAST_CURRENT
		cpfslt	INDF1,0
		return
		incf	INDF1,F,0
		return		
SLOW_STEP
		cpfsgt	VBAT_TEMPH,0
		goto	CHECK_IF_HIGHER
		goto	DECREMENT_CURRENT
CHECK_IF_HIGHER
		movlw	0x06
		movff	PLUSW0,VBAT_TEMPL
		movlw	VBAT42L
		cpfsgt	VBAT_TEMPL,0
		goto	INCREMENT_CURRENT
DECREMENT_CURRENT
		decf	FSR1L,F,0
		movff	INDF1,TEMP_VAR1
		movlw	0x30
		andwf	TEMP_VAR1,F,0
		swapf	TEMP_VAR1,F,0
		tstfsz	TEMP_VAR1,0
		goto	NOT_ZERO
		bsf		INDF1,4,0
		bsf		POSTINC1,5,0
		decf	INDF1,F,0
		return
NOT_ZERO		
		decf	TEMP_VAR1,F,0
		swapf	TEMP_VAR1,F,0
		movlw	0x0C
		addwf	TEMP_VAR1,F,0
		movff	TEMP_VAR1,POSTINC1
		return
INCREMENT_CURRENT		
		cpfslt	VBAT_TEMPL,0
		return
		movlw	FAST_CURRENT
		cpfslt	INDF1,0
		return
		decf	FSR1L,F,0
		movff	INDF1,TEMP_VAR1
		swapf	TEMP_VAR1,F,0
		incf	TEMP_VAR1,F,0
		swapf	TEMP_VAR1,F,0
		btfsc	TEMP_VAR1,6,0
		goto	BIT_NOT_ZERO
		movff	TEMP_VAR1,POSTINC1
		return
BIT_NOT_ZERO
		movlw	0x0C
		movwf	POSTINC1,0
		incf	INDF1,F,0
		return

;******************************************************************************
;CHECK_CHARGE_COMPLETE Subroutine
CHECK_CHARGE_COMPLETE
		movlw	TERMINATION_CURRENT
		cpfsgt	INDF1,0
		goto	CHARGE_END
		movlw	0x04
		movff	PLUSW0,TIME_TEMP
		movlw	ELAPSED_TIME
		cpfslt	TIME_TEMP,0
		goto	CHARGE_END
		return
CHARGE_END
		clrf	POSTDEC1,0				;clear charge current command
		movlw	0x0C
		movwf	INDF1,0					;clear charge current command
		movlw	0x08
		comf	PLUSW0,W,0
		andwf	PORTB,F,0				;turn off
		bcf		INDF0,FAST_CHARGE,0
		bsf		INDF0,CHARGE_COMPLETE,0
		return				

;******************************************************************************
;CHECK_BATTERY_CHARGE Subroutine
CHECK_BATTERY_CHARGE
		movlw	0x08
		movf	PLUSW0,W,0
		xorwf	PORTB,F,0
		movlw	0x05
		movff	PLUSW0,VBAT_TEMPH
		movlw	VBAT40H
		cpfsgt	VBAT_TEMPH,0
		goto	RECHARGE
		movlw	VBAT43H
		cpfslt	VBAT_TEMPH,0
		bsf		INDF0,OVER_VOLTAGE
		return
RECHARGE
		cpfseq	VBAT_TEMPH,0
		goto	RECHARGE_NOW
		movlw	0x06
		movff	PLUSW0,VBAT_TEMPL
		movlw	VBAT40L
		cpfslt	VBAT_TEMPL,0
		return		
RECHARGE_NOW
		movlw	0x08
		movf	PLUSW0,W,0				;move LED mask to working register
		iorwf	PORTC,F,0				;turn on charge LED (green)
		iorwf	PORTB,F,0
		movlw	PRECOND_CURRENT			;set charge current command to 
		movwf	INDF1,0					;preconditioning current
		movlw	0x02
		clrf	PLUSW0					;clear seconds counter
		movlw	0x03
		clrf	PLUSW0					;clear minutes counter
		movlw	0x04
		clrf	PLUSW0					;clear hours counter
		bcf		INDF0,CHARGE_COMPLETE,0
		bsf		INDF0,PRECOND,0
		return
		
;******************************************************************************
;CHECK_FOR_FAULT Subroutine
CHECK_FOR_FAULT
		btfss	CMCON,C1OUT,0
		goto	CHECK_BAY2_OV_FAULT
		btfss	BC_STATUS1,BAT_CONNECT
		goto	CHECK_BAY2_OV_FAULT
		bcf		TRISA,0,0
		bsf		BC_STATUS1,OVER_VOLTAGE,0
CHECK_BAY2_OV_FAULT
		btfss	CMCON,C2OUT,0
		goto	CHECK_TEMPERATURE
		btfss	BC_STATUS2,BAT_CONNECT
		goto	CHECK_TEMPERATURE
		bcf		TRISA,1,0
		bsf		BC_STATUS2,OVER_VOLTAGE,0
CHECK_TEMPERATURE
		btfss	BAY_SELECT,CELL_TEMP_MONITOR,0
		goto	CHECK_TIMER
		movlw	0x07					;copy battery temperature reading to
		movff	PLUSW0,BTEMP_TEMP		;temporary location (BTEMP_TEMP)
		movlw	TEMP_HIGH
		cpfslt	BTEMP_TEMP,0			;battery temperature hot?
		bsf		INDF0,TEMP_FAULT,0		;yes, return temperature fault
		movlw	TEMP_LOW
		cpfsgt	BTEMP_TEMP,0			;battery temperatur cold?
		bsf		INDF0,TEMP_FAULT,0		;yes, return temperature fault
CHECK_TIMER
		btfss	INDF0,PRECOND,0
		return
		movlw	0x03					;copy minutes counter to
		movff	PLUSW0,TIME_TEMP		;temporary location (TIME_TEMP)
		movlw	PRECOND_TIME			
		cpfslt	TIME_TEMP,0				;timer exceeded?
		bsf		INDF0,TIMER_FAULT,0		;yes, return time out fault
		return

;******************************************************************************
;UPDATE_VARIABLES Subroutine
UPDATE_VARIABLES
		btfss	INDF0,BAT_CONNECT,0
		goto	READ_ADC

		bcf		SSPCON1,SSPEN,0
		nop
		bsf		SSPCON1,SSPEN,0
		lfsr	FSR2,0x020
		movlw	PS700READCNT
		movwf	SM_LOOP_CNT,0
		movlw	PS700EEPROM
		movwf	SM_BANK_SEL,0
		call 	GET_PS700_DATA
		btfsc	TMR1H,4,0			;check communications watch-dog
		goto	READ_ADC
		movff	SM_VPACKH,ARG1H
		movff	SM_VPACKL,ARG1L
		movff	SM_VGAINH,ARG2H
		movff	SM_VGAINL,ARG2L
		call	UNSIGNED_16X16_MULTIPLY
		bcf		STATUS,C,0
		rlcf	RES2,F,0
		rlcf	RES3,F,0
		movlw	0x05
		movff	RES3,PLUSW0
		movlw	0x06
		movff	RES2,PLUSW0

		bcf		SSPCON1,SSPEN,0
		nop
		bsf		SSPCON1,SSPEN,0
		lfsr	FSR2,0x02A
		movlw	PS700READCNT
		movwf	SM_LOOP_CNT,0
		movlw	PS700EEPROM
		movwf	SM_BANK_SEL,0
		call 	GET_PS700_DATA
		btfsc	TMR1H,4,0			;check communications watch-dog
		goto	READ_ADC
		movff	SM_TEMPH,ARG1H
		movff	SM_TEMPL,ARG1L
		movff	SM_TGAINH,ARG2H
		movff	SM_TGAINL,ARG2L
		call	UNSIGNED_16X16_MULTIPLY
		bcf		STATUS,C,0
		rrcf	RES3,F,0
		rrcf	RES2,F,0
		btfss	RES3,0,0
		clrf	RES2,0
		movlw	0x07
		movff	RES2,PLUSW0
		goto	UPDATE_TIMERS

READ_ADC
		bcf		SSPCON1,SSPEN,0
		nop
		bsf		SSPCON1,SSPEN,0
		movlw	0x0F
		movff	PLUSW0,ADCON0		;set ADC channel to voltage sense, start conversion
WAIT1	
		btfsc	ADCON0,GO,0			;conversion complete?
		goto	WAIT1				;no, wait
		movff	ADRESH,ARG1H
		movff	ADRESL,ARG1L
		movlw	ADC_VGAINH
		movwf	ARG2H,0
		movlw	ADC_VGAINL
		movwf	ARG2L,0
		call	UNSIGNED_16X16_MULTIPLY
		movlw	0x05
		movff	RES2,PLUSW0			;yes, store result high byte at VBATH 
		movlw	0x06
		movff	RES1,PLUSW0			;store result low byte at VBATL 
		bsf		ADCON0,CHS0,0		;set ADC channel to temperature sense
		bsf		ADCON0,GO,0			;start conversion
WAIT2
		btfsc	ADCON0,GO,0			;conversion complete?
		goto	WAIT2				;no, wait
		movff	ADRESH,ARG1H
		movff	ADRESL,ARG1L
		movlw	ADC_TGAINH
		movwf	ARG2H,0
		movlw	ADC_TGAINL
		movwf	ARG2L,0
		call	UNSIGNED_16X16_MULTIPLY
		movlw	0x07
		subwf	RES1,F,0
		movff	RES1,PLUSW0			;yes, store result at BTEMP
UPDATE_TIMERS
		movlw	0x08
		movf	PLUSW0,W,0			;move LED mask to working register
		btfsc	INDF0,TIMER_FAULT,0	;timer fault?
		xorwf	PORTC,F,0			;yes, flash LED (2S period, 50%)
		btfsc	INDF0,TEMP_FAULT,0	;temperature fault?
		xorwf	PORTC,F,0			;yes, flash LED (2S period, 50%)
		btfsc	INDF0,CHARGE_COMPLETE,0	;charge complete?
		xorwf	PORTC,F,0			;yes, flash LED (2S period, 50%)
		movlw	0x02
		incf	PLUSW0,F,0			;increment second counter
		movff	PLUSW0,TEMP_VAR1
		movlw	0x3C
		cpfseq	TEMP_VAR1,0			;sixty seconds?
		return						;no, ISR complete
		movlw	0x02
		clrf	PLUSW0,0			;yes, clear seconds counter
		movlw	0x03
		incf	PLUSW0,F,0			;increment minutes counter
		movff	PLUSW0,TEMP_VAR1
		movlw	0x3C
		cpfseq	TEMP_VAR1,0			;sixty minutes?
		return						;no, ISR complete
		movlw	0x03
		clrf	PLUSW0,0			;yes, clear minutes counter
		movlw	0x04
		incf	PLUSW0,F,0			;increment hours counter
		return

;******************************************************************************
;GET_PS700_DATA subroutine
GET_PS700_DATA
SEND_WR_START	
		bsf		SSPCON2,SEN,0		;send  start bit		

WR_START_SENT	
		btfss	SSPCON2,SEN,0		;wait for start bit sent
		bra		SEND_WR_ADDRESS
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	WR_START_SENT
		
SEND_WR_ADDRESS	
		bcf		PIR1,SSPIF,0
		nop
		movlw	PS700ADDRESS_W
		movwf	SSPBUF,0

WR_ADDRESS_SENT	
		btfsc	PIR1,SSPIF,0		;wait for address sent
		bra		WR_ADDRESS_ACK
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	WR_ADDRESS_SENT

WR_ADDRESS_ACK	
		nop
		btfss	SSPCON2,ACKSTAT,0	;acknowledgement received
		bra		SEND_COMMANDH		;yes, send command
		btfsc	TMR1H,4,0			;check communications watch-dog
		return

SEND_RESTART	
		bsf		SSPCON2,RSEN		;no, send restart 

RESTART_SENT	
		btfss	SSPCON2,RSEN		;wait for restart sent
		bra		SEND_WR_ADDRESS
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	RESTART_SENT	

SEND_COMMANDH	
		bcf		PIR1,SSPIF,0
		movff	SM_BANK_SEL,SSPBUF
COMMANDH_SENT	
		btfsc	PIR1,SSPIF,0		;wait for command sent
		bra		COMMANDH_ACK
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	COMMANDH_SENT

COMMANDH_ACK	
		nop
		btfss	SSPCON2,ACKSTAT,0	;wait for command acknowledgement
		bra		SEND_COMMANDL
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	COMMANDH_ACK

SEND_COMMANDL	
		bcf		PIR1,SSPIF,0
		movff	POSTINC2,SSPBUF
COMMANDL_SENT	
		btfsc	PIR1,SSPIF,0		;wait for command sent
		bra		COMMANDL_ACK
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	COMMANDL_SENT

COMMANDL_ACK	
		nop
		btfss	SSPCON2,ACKSTAT,0	;wait for command acknowledgement
		bra		SEND_RD_START
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	COMMANDL_ACK
		
SEND_RD_START	
		bsf		SSPCON2,RSEN,0		;send restart for change of direction

RD_START_SENT	
		btfss	SSPCON2,RSEN,0		;wait for restart sent
		bra		SEND_RD_ADDRESS
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	RD_START_SENT
		
SEND_RD_ADDRESS	
		bcf		PIR1,SSPIF,0
		nop
		movlw	PS700ADDRESS_R
		movwf	SSPBUF,0
		
RD_ADDRESS_SENT	
		btfsc	PIR1,SSPIF,0		;wait for read address sent
		bra		RD_ADDRESS_ACK
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	RD_ADDRESS_SENT

RD_ADDRESS_ACK	
		nop
		btfss	SSPCON2,ACKSTAT,0	;wait for read address acknowledgement
		bra		READ_DATA
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	RD_ADDRESS_ACK

READ_DATA	
		bcf		PIR1,SSPIF,0
		bsf		SSPCON2,RCEN,0		;enable receive data

DATA_READ	
		btfsc	PIR1,SSPIF,0		;wait for data recieved
		bra		SEND_NACK
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	DATA_READ

SEND_NACK	
		bsf		SSPCON2,ACKDT,0		;send not acknowledgement
		bsf		SSPCON2,ACKEN,0
		
NACK_SENT	
		btfss	SSPCON2,ACKEN,0		;wait for not acknowldgement sent
		bra		SEND_STOP
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	NACK_SENT

SEND_STOP	
		bsf		SSPCON2,PEN,0		;send stop

STOP_SENT	
		btfss	SSPCON2,PEN,0		;wait for stop sent
		bra		DATA_IN
		btfsc	TMR1H,4,0			;check communications watch-dog
		return
		goto	STOP_SENT

DATA_IN	
		movff	SSPBUF,POSTINC2		;move result
		bcf		SSPCON2,RCEN,0		;turn off receive data enable

		decfsz	SM_LOOP_CNT,F,0
		goto	CHECK_FOR_REGISTER_READ
		return

CHECK_FOR_REGISTER_READ
		btfsc	SM_LOOP_CNT,2,0
		goto	SEND_WR_START
		movlw	PS700REGISTERS
		movwf	SM_BANK_SEL
		goto	SEND_WR_START	
		
;******************************************************************************
;UNSIGNED_16X16_MULTIPLY subroutine
UNSIGNED_16X16_MULTIPLY
		movf	ARG1L,W,0
		mulwf	ARG2L,0
		movff	PRODH,RES1
		movff	PRODL,RES0
		movf	ARG1H,W,0
		mulwf	ARG2H,0
		movff	PRODH,RES3
		movff	PRODL,RES2
		movf	ARG1L,W,0
		mulwf	ARG2H,0
		movf	PRODL,W,0
		addwf	RES1,F,0
		movf	PRODH,W,0
		addwfc	RES2,F,0
		clrf	WREG,0
		addwfc	RES3,F,0
		movf	ARG1H,W,0
		mulwf	ARG2L,0
		movf	PRODL,W,0
		addwf	RES1,F,0
		movf	PRODH,W,0
		addwfc	RES2,F,0
		clrf	WREG,0
		addwfc	RES3,F,0
		return

;******************************************************************************
;End of program

		END
